package cn.iocoder.yudao.framework.excel.util;

import cn.iocoder.yudao.framework.excel.handler.WaterMarkHandler;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;

/**
 * Excel 工具类
 *
 * @author 捷道源码
 */
public class ExcelUtils {

    /**
     * 将列表以 Excel 响应给前端
     *
     * @param response  响应
     * @param filename  文件名
     * @param sheetName Excel sheet 名
     * @param head      Excel head 头
     * @param data      数据列表哦
     * @param <T>       泛型，保证 head 和 data 类型的一致性
     * @throws IOException 写入失败的情况
     */
    public static <T> void write(HttpServletResponse response, String filename, String sheetName,
                                 Class<T> head, List<T> data) throws IOException {
        // 输出 Excel
        EasyExcel.write(response.getOutputStream(), head)
                .inMemory(true)
                .autoCloseStream(false) // 不要自动关闭，交给 Servlet 自己处理
//                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy()) // 基于 column 长度，自动适配。最大 255 宽度
                .registerWriteHandler(new WaterMarkHandler("捷道机密文件，禁止传播"))
                .sheet(sheetName).doWrite(data);
        // 设置 header 和 contentType。写在最后的原因是，避免报错时，响应 contentType 已经被修改了
        response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filename, "UTF-8"));
        response.setContentType("application/vnd.ms-excel;charset=UTF-8");
    }


    /**
     * 将列表以 Excel文件存储到临时目录
     *
     * @param dir          目录
     * @param filename      文件名(尽量避免文件名称重复)
     * @param sheetName     Excel sheet 名
     * @param head          Excel head 头
     * @param data          数据列表哦
     * @param <T>           泛型，保证 head 和 data 类型的一致性
     * @throws IOException 写入失败的情况
     */
    public static <T> String write(String dir, String filename, String sheetName,
                                   Class<T> head, List<T> data) throws IOException {
        // 创建需要写入的文件目录
        File fileDir = new File(dir);
        if (!fileDir.exists()){
            // 不存在则创建一个目录
            fileDir.mkdirs();
        }
        // 可能由名称重复的文件，所以需要
        String path = dir + filename;
        OutputStream outputStream = new FileOutputStream(path);
        // 输出 Excel
        EasyExcel.write(outputStream, head)
                .inMemory(true)
                .autoCloseStream(true) // 不要自动关闭，交给 Servlet 自己处理
//                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy()) // 基于 column 长度，自动适配。最大 255 宽度
                .registerWriteHandler(new WaterMarkHandler("捷道机密文件，禁止传播"))
                .sheet(sheetName).doWrite(data);
        return path;
    }

    public static <T> List<T> read(MultipartFile file, Class<T> head) throws IOException {
        return EasyExcel.read(file.getInputStream(), head, null)
                .autoCloseStream(false)  // 不要自动关闭，交给 Servlet 自己处理
                .doReadAllSync();
    }

}
